@@ -4,29 +4,35 @@ package Makefile;
 our $VERSION = 0.1;
 
 use strict;
-use Data::Dumper;
+use Config;
+use Carp;
 use Storable qw/dclone/;
+require Exporter;
+use Data::Dumper;
 
 $Data::Dumper::Purity=1;
 $Data::Dumper::Deparse=1;
 $Data::Dumper::Indent=1;
 
-require Exporter;
 
 push @{Makefile::ISA}, 'Exporter';
-@{Makefile::EXPORT_OK} = qw/get_version path_copy file_copy rmdir_force dir_copy pd_read pd_write get_version_from compare_version/;
+@{Makefile::EXPORT_OK} = qw/
+	get_version get_version_from compare_version
+	path_copy file_copy
+	rmdir_force dir_copy pd_read chdir
+	pd_write
+/;
 
 our %default_config = (
 	'default_target' => 'all',
-	'log_file' => 'm/make.log',
 	'conf_file' => 'm/config.pd',
 	'makefile' => 'Makefile',
 	'target_dir' => 'm/targets',
 	'manifest' => 'MANIFEST',
 	'tools' => {
 		# name => [ command, [extensions] ]
-		'perl' => [ '/usr/bin/perl', ['pl', 'PL', 'pm'] ],
-		'shell' => [ '/bin/sh' , [ 'sh' ] ],
+		'perl' =>  [ $Config{perl5}, ['pl', 'PL', 'pm'] ],
+		'shell' => [ $Config{sh},    [ 'sh' ]           ],
 	}
 );
 
@@ -59,7 +65,7 @@ sub new {
 	$self->{tools} = \%tmp1;
 	#</VUNZIG>
 
-	if ($0 ne 'Makefile.PL') {
+	if ($0 !~ /(Makefile\.PL|configure)$/) {
 		if (@var_arg) {# reverse serialised vars
 			for (0..$#{$self->{var_order}}) { $self->{vars}{$self->{var_order}[$_]} = $var_arg[$_] || ''; }
 		}
@@ -83,16 +89,14 @@ sub new {
 				}
 			}
 		}
-		$self->truncate_log;
 	}
-	$self->print_log_header("Script: $0");
 
 	return $self;
 }
 
 sub pd_read {
 	my $file = pop;
-	open FILE, "<$file" || die $!;
+	open FILE, "<$file" || die "==> Could not open file $file\n";
 	my $cont = join("",(<FILE>));
 	close FILE;
 	my $VAR1;
@@ -104,29 +108,28 @@ sub pd_read {
 sub pd_write {
 	my $ref = pop;
 	my $file = pop;
-	open FILE, ">$file" || die $!;
+	open FILE, ">$file" || die "==> Could not open file $file\n";
 	print FILE Dumper($ref);
-	close FILE;
+	close FILE || die "==> Could not write file $file\n";
 }
 
 sub check_manifest {
 	my $self = shift;
 	$self->{manifest_ok} = 1;
+	print "debug: using manifest: $self->{manifest}\n";
 	for ($self->manifest) {
 		if ($self->{vars}{VERBOSE}) { print "Checking for file \"$_\".\n"; }
 		unless (-f $_) {
 			$self->{manifest_ok} = 0;
 			print STDERR "==> The file \"$_\" seems to miss from your package.\n";
-			$self->print_log( "The file \"$_\" seems to miss from your package.", 'error');
 		}
 	}
 	if ($self->{manifest_ok}) {
 		print "Manifest seems to be ok.\n";
-		$self->print_log("Manifest seems to be ok.", "CHECK");
 		return 1;
 	}
 	else { 
-		print "==> This package seems to be incomplete, maybe you should fetch a new source tree.\n";
+		print "This package seems to be incomplete, maybe you should fetch a new source tree.\n";
 		return 0;
 	}
 }
@@ -136,11 +139,11 @@ sub manifest {
 	my $self = shift;
 	my $man = shift || $self->{manifest};
 	my @files = ();
-	open IN, $man || die $!;
-	while (<IN>) {
-		$_ = m/^\s*(.+?)\s/;
-		unless ($1) { next; }
-		push @files, $1;
+	open IN, $man || die "==> Could not open $man\n";
+	while (<IN>) { # FIXME what about names with whitespaces etc. ?
+		next if m/^\s*#/;
+		m/^\s*(.+?)(\s+#|$)/;
+		push @files, $1 if $1;
 	}
 	close IN;
 	return @files;
@@ -153,19 +156,21 @@ sub check_dep {
 	while (my ($module, $version) = each %{$self->{include}{PREREQ_PM}}) {
 		my $our_v = get_version($module);
 		if ($self->{vars}{VERBOSE}) { print "checking $module version $version we have: ".$our_v."\n"; }
-		unless (compare_version($version, $our_v) >= 0) {
-			$self->{dep_ok} = 0;
-			print STDERR "==> Failed dependency \"$module\"".($version ? " version $version\n" : "\n");
-			$self->print_log("Failed dependency \"$module\"".($version ? " version $version" : ""), 'error');
+		unless (defined $our_v && compare_version($version, $our_v) >= 0) {
+			$self->{dep_ok} = 0 unless $our_v;
+			print STDERR
+				( $our_v ? 'Warning :' : '==> Failed'),
+				qq{ dependency "$module" },
+				($version ? "version $version" : ''),
+				($our_v ? " suggested - we have $our_v\n" : "\n");
 		}
 	}
 	if ($self->{dep_ok}) {
 		print "Dependencies seem to be ok.\n";
-		$self->print_log("Dependencies seem to be ok.", "CHECK");
 		return 1;
 	}
 	else {
-		print "==> Some dependecies failed, you should install these packages first.\n";
+		print "Some dependecies failed, you should install these packages first.\n";
 		return 0;
 	}
 }
@@ -218,7 +223,7 @@ sub write_makefile {
 	$self->{target_names} =  [ map {$_->[0]} @targets ];
 
 	#if (-e $self->{makefile}) { unlink $self->{makefile}; }
-	open FILE, '>'.$self->{makefile} || die $!;
+	open FILE, '>'.$self->{makefile} || die '==> Could not open '.$self->{makefile}."\n";
 	print FILE "## This Makefile was autogenerated by Makefile.pm\n## Dont try to change anything here, change the perl scripts\n\n";
 
 	print FILE "#--[ Included data ]--[ some of this might be used by CPAN ]\n\n";
@@ -233,7 +238,9 @@ sub write_makefile {
 	print FILE "\n#--[ Phony targets ]--[ targets dir is \"$self->{target_dir}\" ]\n\n";
 	print FILE '.PHONY : '.join(' ', (map {$_->[0]} @targets))."\n\n";
 
-	my $arg = join(' ', map {'"$('.uc($_).')"'} @{$self->{var_order}});
+	#my $arg = join(' ', map {'"$('.uc($_).')"'} @{$self->{var_order}});
+	print FILE '_ARGS = '.join(' ', map {'"$('.uc($_).')"'} @{$self->{var_order}})."\n\n";
+	my $arg = '$(_ARGS)';
 
 	foreach my $t (@targets) {
 		my $dep = $self->{depends}{$t->[0]} ? join(' ', @{$self->{depends}{$t->[0]}}) : '' ;
@@ -244,6 +251,7 @@ sub write_makefile {
 	}
 
 	my $my_file = __FILE__;
+	my $my_mani = $self->{manifest};
 
 	print FILE << "END"
 #--[ basic check ]
@@ -255,7 +263,7 @@ Makefile : Makefile.PL $my_file
 
 END
 ;
-	close FILE;
+	close FILE || die '==> Could not write'.$self->{makefile}."\n";
 
 	my $spec = '...';
 	if ($self->{include}{NAME}) {
@@ -291,10 +299,10 @@ sub print_usage {
 	my $name = $self->{include}{NAME} || 'unknown';
 	my $version = $self->{include}{VERSION} || 'unknown';
 	my $author = $self->{include}{AUTHOR} || 'unknown';
-	print << "END"
+	print << "END";
 
-## This package uses Makefile.pm -- yet another approach to Makefile.PL
-## by Jaap Karssenberg || Pardus [Larus] --=-- 2003
+## This package uses Makefile.pm - yet another approach to Makefile.PL
+## by Jaap Karssenberg || Pardus [Larus] -=- 2003
 
 --[ Package information ]
 
@@ -313,59 +321,13 @@ sub print_usage {
 --[ Variables ]
 
 END
-	;
-	for (keys %{$self->{vars}}) { print '  '.$_.($self->{vars}{$_} ? ' :  '.$self->{vars}{$_} : '')."\n"; }
+
+	for (keys %{$self->{vars}}) { 
+		print '  ', $_, ($self->{vars}{$_} ? ' :  '.$self->{vars}{$_} : ''), "\n"; 
+	}
 	print "\n\n";
 }
 
-sub truncate_log {
-	my $self = shift;
-	my $name = $self->{include}{NAME} || 'unknown';
-	my $version = $self->{include}{VERSION} || 'unknown';
-	my $author = $self->{include}{AUTHOR} || 'unknown';
-	if ($self->{log_file}) { #print "log file: $self->{log_file}\n";
-		open L, '>'.$self->{log_file} || die $!;
-		print L << "END"
-# Log file for the make/compile/test/install process
-
-## This package uses Makefile.pm -- yet another approach to Makefile.PL
-## by Jaap Karssenberg || Pardus [Larus] --=-- 2003
-
---[ Package information ]
-
- NAME    : $name
- VERSION : $version
- AUTHOR  : $author
-END
-	;
-	close L;
-	}
-}
-
-sub print_log_header {
-	my $self = shift;
-	my $text = shift;
-	if ($self->{log_file}) {
-		open L, '>>'.$self->{log_file};
-		print L "\n--[ $text ]\n\n";
-		close L;
-	}
-}
-
-sub print_log {
-	my $self = shift;
-	my $text = shift;
-	my $type = shift || 'MESS';
-	if ($self->{log_file}) { #print "debug got: ".uc($type).": $text -- for log file $self->{log_file}\n";
-		unless (-f $self->{log_file}) { die "Could not find log file" }
-		open L, '>>'.$self->{log_file} || die $!;
-		print L uc($type).": $text\n";
-		close L;
-	}
-}
-
-## helpfull class routines
-
 sub compare_version {
 	my $gr = pop || 0;
 	my $ls = pop || 0;
@@ -391,37 +353,59 @@ sub compare_version {
 }
 
 sub get_version {
-	my $n = pop @_;
-	$n =~ s/\(.*\)$//;
-	my $c = $n.'.pm';
-	$c =~ s{::}{/}g;
-	my $version = undef;
-	eval qq{
-		require \"$c\";
-		\$version = \$$n\:\:VERSION;
-	};
-	return $version;
-}
-
-sub get_version_from {
-	my $f = pop @_;
-	my $version = undef;
-	open PACK, $f || die $!;
-	while (<PACK>) {
-		if(/([\$*])(([\w\:\']*)\bVERSION)\b.*\=/) { # regex from MakeMaker docs
-			$version = eval($_);
+	my $name = pop @_;
+	my $file;
+	unless ($name =~ m{/}) {
+		$name =~ s/\(.*\)$//;
+		$file = $name.'.pm';
+		$file =~ s{::}{/}g;
+	}
+	else { $file = $name }
+	if ($INC{$file}) { $file = $INC{$file} }
+	else {
+		for (@INC) { # get absolute path
+			next unless -f $_.'/'.$file;
+			$file = $_.'/'.$file;
 			last;
 		}
 	}
-	close PACK;
-	return $version;
+	return undef unless -f $file;
+	return get_version_from($file) || 0;
+}
+
+sub get_version_from {
+	# Code borrowed from ExtUtils::MM_Unix
+	my $file = pop;
+	my $inpod = 0;
+	my $result = undef;
+	open IN, $file || die "==> Could not open file $file\n";
+	while (<IN>) {
+		$inpod = /^=(?!cut)/ ? 1 : /^=cut/ ? 0 : $inpod;
+		next if $inpod || /^\s*#/;
+		next unless /([\$*])(([\w\:\']*)\bVERSION)\b.*\=/;
+		my $eval = qq{
+			package Makefile::_version;
+			no strict;
+
+			local $1$2;
+			\$$2=undef; 
+			$_
+			\$$2
+        	};
+		local $^W = 0;
+		$result = eval($eval);
+		print STDERR "Warning: Could not eval '$eval' in $file: $@" if $@;
+		last;
+	}
+	close IN || die "==> Could not read file $file\n";
+	return $result;
 }
 
 sub path_copy {
 	my $e = 'usage: path_copy $from, $to';
 	my $to = pop @_ || die $e;
 	my $from = pop @_ || die $e;
-	unless (-e $from) { die "no such thing \"$from\" to copy"; }
+	unless (-e $from) { confess "no such thing \"$from\" to copy"; }
 
 	# create dir tee
 	if ($to =~ m/^(.*)\/(.*)/) {
@@ -429,7 +413,7 @@ sub path_copy {
 		my $dir = '';
 		while (@tree) {
 			$dir .= (shift @tree).'/';
-			unless (-d $dir) { mkdir $dir; }
+			unless (-d $dir) { mkdir $dir }
 		}
 	}
 
@@ -442,15 +426,16 @@ sub file_copy {
 	my $from = pop @_ || die $e;
 	unless (-e $from) { die "no such thing \"$from\" to copy"; }
 
-    if (([stat$from]->[9]==[stat$to]->[9])&&(-s$from==-s$to)) {
-        return 1;
-    }
+	if (
+		( [stat $from]->[9] == [stat $to]->[9] )
+		&& ( -s $from == -s $to )
+	) { return 1 }
         
-	open IN, $from || die $!;
-	open OUT, ">$to" || die $!;
+	open IN, $from || die "==> Could not open file $from\n";
+	open OUT, ">$to" || die "==> Could not open file $to\n";
 	while (<IN>) { print OUT $_; }
 	close IN;
-	close OUT;
+	close OUT || die "==> Could not write file $to\n";
     # set new file's mtime
     utime(@{[stat$from]}[8,9],$to);
 	return 1;
@@ -460,10 +445,10 @@ sub rmdir_force {
 	my $dir = pop || die 'usage: rmdir_force $dir;';
 	$dir =~ s/\/$//; #/
 	unless (-e $dir) { return 1; }
-	opendir D, $dir || die $!;
+	opendir D, $dir || die "==> Could not open dir $dir\n";
 	my @dinge = grep {$_ !~ m/^\.\.?$/} readdir D;
 	closedir D;
-	for (grep { !(-d $dir.'/'.$_) } @dinge) { unlink $dir.'/'.$_ || die $! }
+	for (grep { !(-d $dir.'/'.$_) } @dinge) { unlink $dir.'/'.$_ || die "==> Could not remove $dir/$_\n" }
 	for (grep { -d $dir.'/'.$_ } @dinge) { rmdir_force($dir.'/'.$_) } # recurs
 	return rmdir $dir;
 }
@@ -476,9 +461,9 @@ sub dir_copy {
 
 	$from =~ s/\/?$/\//;
 	$to =~ s/\/?$/\//;
-	unless (-e $to) { mkdir($to) || die $!; }
+	unless (-e $to) { mkdir($to) || die "Could not make dir $to\n"; }
 
-	opendir FROM, $from || die $!;
+	opendir FROM, $from || die "Could not open dir $from\n";
 	my @files = grep {$_ !~ m/^\.\.?$/} readdir FROM;
 	closedir FROM;
 
@@ -498,7 +483,7 @@ __END__
 
 =head1 NAME
 
-Makefile -- yet another approach to Makefile.PL
+Makefile - yet another approach to Makefile.PL
 
 =head1 SYNOPSIS
 
@@ -558,9 +543,8 @@ One can add make targets by simply adding scripts or directories to the target d
 
 For obvious reasons it is not possible to have a target called 'CVS' --
 'cvs' or any combination with it should work fine. Also it is not wise to call a target
- 'Makefile' or something alike. Since Makefile.pm relies on @ARGV one should not touch
- @ARGV in a target script or at least do C<require 'm/Makefile.pm';> first.
-
+'Makefile' or something alike. Since Makefile.pm relies on @ARGV one should not touch
+@ARGV in a target script or at least do C<require 'm/Makefile.pm';> first.
 
 Also since the module itself just glues scripts in the Makefile, it can also be used for
 all kind of packages other then perl extensions.
@@ -594,11 +578,6 @@ you should also modify all target scripts form C<Makefile->new> to C<Makefile->n
 
 The name of the created makefile, defaults to 'Makefile'.
 
-=item log_file
-
-Log file for the make process, defaults to F<m/make.log>.
-If empty string no log is created.
-
 =item target_dir
 
 Dir to scan for targets, defaults to 'm/targets'. All files and dirs in this dir will be regarded as
@@ -625,7 +604,7 @@ depends on. Can be used to bundle targets.
 =item vars
 
 Vars should be a hash ref with of the form C<{ var_name => default_value, }>. Values
-should always be strings. Var names will be translated to all caps -- to acces them from
+should always be strings. Var names will be translated to all caps - to acces them from
 targets scripts use this last form. The var "VERBOSE" is special.
 
 Vars can be commandline args to make like: C<make test TEST_VERBOSE=1> and the defaults can be
@@ -661,7 +640,7 @@ Entries have their var name as key (like vars this is set to caps) and as value
 interpreter binary location and a array of possible file extensions. This will asociate these extensions 
 with the given interpreter and allows you to supply alternate interpreter locations as commandline arguments.
 
-This hash defaults to C<{ 'PERL' => [ '/usr/bin/perl', ['pl', 'PL', 'pm'] ], 'SHELL' => [ '/bin/sh', ['sh'] ], }>
+This hash defaults to C<{ 'PERL' => [ $Config{perl5}, ['pl', 'PL', 'pm'] ], 'SHELL' => [ $Config{sh}, ['sh'] ], }>
 
 =back
 
@@ -669,75 +648,68 @@ This hash defaults to C<{ 'PERL' => [ '/usr/bin/perl', ['pl', 'PL', 'pm'] ], 'SH
 
 =over 4
 
-=item new(\%config)
+=item C<new(\%config)>
 
 Simple constructor, takes config hashref. If no config is supplyed
 tries to read the default config file and/or uses default config.
 
 Most likely one only supplies config from the F<Makefile.PL>.
 
-Also this method truncates the log file when called from F<Makefile.PL>.
-
-=item pd_read($file)
+=item C<pd_read($file)>
 
 Eval $file and return $VAR1 .
 
-=item pd_write($file, \%hash)
+=item C<pd_write($file, \%hash)>
 
 Print hash to file with Data::Dumper .
 
-=item check_manifest()
+=item C<check_manifest()>
 
 Check for missing files, prints errors and returns bit.
 
-=item manifest()
+=item C<manifest()>
 
 Reads the manifest and returns a array filenames.
 
-=item check_dep()
+=item C<check_dep()>
 
 Check for missing dependencies, prints errors and returns bit.
 
-=item write_makefile()
+=item C<write_makefile()>
 
 Print makefile. This does _not_ create something like
 a Makefile.old -- if you want this do it yourself from Makefile.PL .
 Also this method writes a fresh config file used as
 a bypass around the Makefile :]
 
-=item print_log($text, $type)
+=item C<compare_version($v1, $v2)> [non oo method]
 
-Print entry to log file (if there is one) of the format C<TYPE: text>.
-
-=item compare_version($v1, $v2) [non oo method]
-
-Returns 0 if $v1 eq $v2, 1 if $v1 < $v2
-and -1 if $v1 > $v2.
+Returns 0 if $v1 eq $v2, 1 if $v1 < $v2 and -1 if $v1 > $v2.
 Used to compare required versions to installed versions.
 
-=item get_version($class_name) [non oo method]
+=item C<get_version($class_name)> [non oo method]
 
 Get version of an installed module.
 
-=item get_version_from($file) [non oo method]
+=item C<get_version_from($file)> [non oo method]
 
 Get version declared in file. This uses the same regex as L<ExtUtils::MakeMaker> uses for
 "VERSION_FROM" this is: C</([\$*])(([\w\:\']*)\bVERSION)\b.*\=/> . See docs MM for more details.
 
-=item path_copy($from, $to) [non oo method]
+=item C<path_copy($from, $to)> [non oo method]
 
 Copy file $from to file $to, create directories if needed.
 
-=item file_copy($from, $to) [non oo method]
+=item C<file_copy($from, $to)> [non oo method]
 
 Copy file $from to file $to, create directories if needed.
 
-=item rmdir_force($dir) [non oo method]
+=item C<rmdir_force($dir)> [non oo method]
 
 Like C<rm -fr dir>, used to rm non-empty dir trees.
 I<Use with care>.
 
-=item dir_copy($from, $to) [non oo method]
+=item C<dir_copy($from, $to)> [non oo method]
 
 Copy contents of dir $from to dir $to
 
@@ -755,21 +727,26 @@ It doesn't have all the fancy features like XS support, this should be implement
 the module is just glue. On the other hand standard routines could be put in the module.
 Write your own fancy script or use MakeMaker instead.
 
-It depends on other modules -- like Data::Dumper. Also the scripts use other
+It depends on other modules, see L</DEPENDS> . Also the scripts use other
 packages like Pod::Man. This might be a problem in some situations.
 
 I have no idea about the portability to other platforms, but one can just patch the scripts.
 At least it should use File::Spec but it doesn't.
 
+=head1 DEPENDS
+
+ Carp
+ Config
+ Exporter
+ Strorable
+ Data::Dumper
+
 =head1 HISTORY
 
 This module was born as part of the zoidberg project out of a personal itch with MakeMaker.
 Features where selected upon the needs of this project. See the Zoidberg for an extensive example.
 
-I<"You all _do_ write docs, don't you?"> -- Schwern on Yapc::Europe::2002
-
-Well, next time ask yourself, are these docs comprehensible, complete and simple enough for the average coder?
-Also one example might do more good then any number of lines description.
+I<"You all _do_ write docs, don't you?"> - Schwern on Yapc::Europe::2002
 
 =head1 AUTHOR
 
